@@ -58,8 +58,8 @@ Rcpp::List RInterfaceCV(arma::mat& x, arma::vec& y,
   // Vector for trimming grid of the fold
   arma::uvec h_fold = arma::uvec(h.n_elem);
   
-  // Multithreading over the folds of the data
-  // # pragma omp parallel for num_threads(n_threads)
+  // FIXED: Enable parallelization over folds
+#pragma omp parallel for num_threads(n_threads) 
   for (arma::uword fold = 0; fold < n_folds; fold++) {
     
     // Creating the training and test data
@@ -81,15 +81,15 @@ Rcpp::List RInterfaceCV(arma::mat& x, arma::vec& y,
     arma::mat mad_x_ensemble = MedianAbsoluteDeviationEnsemble(mad_x, n_models);
     double mad_y = MedianAbsoluteDeviation(y_train);
     
-    // Creation of 3D vector to store ensembles
-    std::vector<std::vector<std::vector<EnsembleModel>>> ensembles;
+    // MEMORY FIX: Create fold-specific ensembles (will be destroyed at end of loop iteration)
+    std::vector<std::vector<std::vector<EnsembleModel>>> fold_ensembles;
     
     // Trimming grid for the fold
     for(arma::uword h_id = 0; h_id < h_fold.n_elem; h_id++)
       h_fold[h_id] = std::round((double)n_in_fold / n * h[h_id]);
-
-    // Initialization of ensembles
-    InitializeEnsembleModel(ensembles,
+    
+    // Initialization of ensembles for this fold only
+    InitializeEnsembleModel(fold_ensembles,
                             x_train, y_train,
                             med_x_data, mad_x_data,
                             med_x_ensemble, mad_x_ensemble,
@@ -100,19 +100,24 @@ Rcpp::List RInterfaceCV(arma::mat& x, arma::vec& y,
                             max_iter,
                             initial_split_folds.slice(fold));
     
-    // Neighborhood search
+    // Neighborhood search for this fold
     if (neighborhood_search)
-      NeighborhoodSearch(ensembles,
+      NeighborhoodSearch(fold_ensembles,
                          h, t, u,
                          p, n_models,
                          neighborhood_search_tolerance);
     
-    // Storing the prediction residuals of the CV procedure
-    for (arma::uword h_ind = 0; h_ind < h.size(); h_ind++)
-      for (arma::uword t_ind = 0; t_ind < t.size(); t_ind++)
-        for (arma::uword u_ind = 0; u_ind < u.size(); u_ind++) 
-          prediction_residuals[h_ind][t_ind][u_ind](test) = gamma * ensembles[h_ind][t_ind][u_ind].Prediction_Residuals_Ensemble(x_test, y_test) +
-            (1 - gamma) * ensembles[h_ind][t_ind][u_ind].Prediction_Residuals_Models(x_test, y_test);
+    // CRITICAL SECTION: Store prediction residuals (needs to be thread-safe)
+{
+  for (arma::uword h_ind = 0; h_ind < h.size(); h_ind++)
+    for (arma::uword t_ind = 0; t_ind < t.size(); t_ind++)
+      for (arma::uword u_ind = 0; u_ind < u.size(); u_ind++) 
+        prediction_residuals[h_ind][t_ind][u_ind](test) = 
+          gamma * fold_ensembles[h_ind][t_ind][u_ind].Prediction_Residuals_Ensemble(x_test, y_test) +
+          (1 - gamma) * fold_ensembles[h_ind][t_ind][u_ind].Prediction_Residuals_Models(x_test, y_test);
+}
+
+// fold_ensembles goes out of scope here and memory is freed
   }
   
   //_____________________________
@@ -129,10 +134,10 @@ Rcpp::List RInterfaceCV(arma::mat& x, arma::vec& y,
   arma::mat mad_x_ensemble = MedianAbsoluteDeviationEnsemble(mad_x, n_models);
   double mad_y = MedianAbsoluteDeviation(y);
   
-  // Creation of 3D vector to store ensembles
+  // Creation of 3D vector to store ensembles on full data
   std::vector<std::vector<std::vector<EnsembleModel>>> ensembles;
   
-  // Initialization of ensembles
+  // Initialization of ensembles on full data
   InitializeEnsembleModel(ensembles,
                           x, y,
                           med_x_data, mad_x_data,
@@ -144,7 +149,7 @@ Rcpp::List RInterfaceCV(arma::mat& x, arma::vec& y,
                           max_iter,
                           initial_split);
   
-  // Neighborhood search
+  // Neighborhood search on full data
   if (neighborhood_search)
     NeighborhoodSearch(ensembles,
                        h, t, u,
